import libevdev
import pydrm

from queue import Queue
from threading import Thread

bytes_per_pixel = 4 # XRGB
line_width = 2
pen_event = '/dev/input/event4' # chosen by fair dice roll.

X = libevdev.EV_ABS.ABS_X
Y = libevdev.EV_ABS.ABS_Y
axes = (X, Y)

drm = pydrm.SimpleDrm()
drm.draw.rectangle(((0, 0), (drm.framebuffer.width, drm.framebuffer.height)), fill='white', width=0)
drm.flush()
resolution = {
    X: drm.framebuffer.width,
    Y: drm.framebuffer.height,
}

points = Queue()

def render():
    while (point := points.get()):
        corner = (point[0] + line_width, point[1] + line_width)
        for y in range(point[1], corner[1]):
            begin = (y * drm.framebuffer.width +  point[0]) * bytes_per_pixel
            end   = (y * drm.framebuffer.width + corner[0]) * bytes_per_pixel
            # Skip image rendering and update the framebuffer directly
            drm.framebuffer.bo.map[begin:end] = bytes(end - begin)
        pydrm.framebuffer.DrmFramebuffer.flush(drm.framebuffer, point[0], point[1], corner[0], corner[1])

try:
    render_thread = Thread(target=render)
    render_thread.start()

    with open(pen_event, 'rb') as pen_fd:
        pen = libevdev.Device(pen_fd)
        print(pen.name)

        contact = False
        last_point = None
        position = {}
        scale = {axis: resolution[axis] / pen.absinfo[axis].maximum for axis in axes}
        print(scale)

        while True:
            for e in pen.events():
                if e.matches(libevdev.EV_ABS.ABS_PRESSURE):
                    contact = bool(e.value)
                    continue
                if not contact:
                    continue
                if e.code in axes:
                    position[e.code] = e.value
                elif e.matches(libevdev.EV_SYN) and X in position and Y in position:
                    point = tuple(int(position[axis] * scale[axis] / line_width) * line_width for axis in axes)
                    if point != last_point:
                        last_point = point
                        points.put(point)

finally:
    points.put(None)
    render_thread.join()
